@@ -35,7 +35,7 @@ type xmltree =
 let string_of_xml ?ns_prefix ?indent tree =
   try
     let b = Buffer.create 256 in
-    let output = Xmlm.make_output ?ns_prefix ~indent ~decl: false (`Buffer b) in
+    let output =Xmlm.make_output ?ns_prefix ~indent ~decl: false (`Buffer b) in
     let frag = function
     | E (tag, childs) -> `El (tag, childs)
     | D d -> `Data d
@@ -55,12 +55,36 @@ let source_string = function
 | `Channel _ | `Fun _ -> ""
 ;;
 
+(* [trim] borrowed from [String] to ensure that this code also
+   compiles with OCaml 3.12. *)
+let is_space = function
+  | ' ' | '\012' | '\n' | '\r' | '\t' -> true
+  | _ -> false
+
+let trim s =
+  let len = String.length s in
+  let i = ref 0 in
+  while !i < len && is_space (String.unsafe_get s !i) do
+    incr i
+  done;
+  let j = ref (len - 1) in
+  while !j >= !i && is_space (String.unsafe_get s !j) do
+    decr j
+  done;
+  if !i = 0 && !j = len - 1 then
+    s
+  else if !j >= !i then
+    String.sub s !i (!j - !i + 1)
+  else
+    ""
 
 let xml_of_source source =
   try
-    let input = Xmlm.make_input ~strip: true ~enc: (Some `UTF_8) source in
+    let input = Xmlm.make_input ~strip:false ~enc: (Some `UTF_8)
+      (*~entity: (fun s -> Some s)*) source
+    in
     let el tag childs = E (tag, childs)  in
-    let data d = D d in
+    let data d = D (trim d) in
     let (_, tree) = Xmlm.input_doc_tree ~el ~data input in
     tree
   with
@@ -91,9 +115,10 @@ exception Error of string;;
 let error msg = raise (Error msg);;
 
 let find_ele name e =
-    match e with
-      E ((("",e),_),_) when name = String.lowercase e -> true
-    | _ -> false
+  match e with
+    E ((("",e),_),_) when name = String.lowercase e -> true
+  | E ((("http://purl.org/rss/1.0/",e),_),_) when name = String.lowercase e -> true
+  | _ -> false
 
 let apply_opt f = function
     None -> None
@@ -515,19 +540,15 @@ let channel_of_source opts source =
              with
                Not_found -> failwith "Parse error: no channel"
             )
-      |	"rdf:rdf" ->
+        |	"rdf" ->
             (
-             match subs with
-             | [] ->
-                 failwith "Parse error: no channel"
-             | (E ((("",e), atts), subs)) :: q ->
-                 (
-                  match String.lowercase e with
-                    "channel" ->
-                      (channel_of_xmls opts (subs @ q), opts.errors)
-                  | _ -> failwith "Parse error: not channel"
-                 )
-             | _ ->
+             try
+               let elt =  List.find (find_ele "channel") subs in
+               match elt with
+               | E ((_, atts), subs_ch)->
+                   (channel_of_xmls opts (subs_ch @ subs), opts.errors)
+               | _ -> assert false
+             with Not_found ->
                  failwith "Parse error: not channel"
             )
         |	_ ->
@@ -535,7 +556,7 @@ let channel_of_source opts source =
       in
       let namespaces =
          let f ((prefix, name), value) acc =
-           if prefix = Xmlm.ns_xmlns then
+           if prefix = Xmlm.ns_xmlns && name <> "xmlns" then
              (name, value) :: acc
           else
             acc
